from setuptools import setup

setup(name='otumba',
      version='0.2.2',
      description='AI on Cloud Distributed Systems',
      packages=['otumba'],
      author = 'Pedro Coronel Santeliz',
      author_email = 'pelsanmexico@gmail.com',
      zip_safe=False)
